"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const Rx = __importStar(require("rxjs"));
const operators_1 = require("rxjs/operators");
const webSocket_1 = require("rxjs/webSocket");
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const types_1 = require("../../types");
const debug = __importStar(require("../../debug"));
const common_1 = require("./common");
const cleanEvents_1 = require("../../utils/cleanEvents");
const wethToEth_1 = require("../../utils/wethToEth");
const subscribeMessage = (options) => {
    const base = wethToEth_1.wethToEth(options.pair.base.symbol);
    const quote = wethToEth_1.wethToEth(options.pair.quote.symbol);
    const message = {
        event: 'subscribe',
        channel: 'book',
        symbol: `t${base}${quote}`,
        prec: 'R0',
    };
    return message;
};
const getWebsocketUrl = (options) => {
    switch (options.network) {
        case types_1.Network.MAINNET:
            return 'wss://api.ethfinex.com/ws/2';
        case types_1.Network.KOVAN:
            return 'wss://kovan.api.ethfinex.com/ws/2';
        default:
            throw new Error('Invalid network.');
    }
};
const normalizeOrderEvent = (options, order) => {
    const [, price] = order;
    const id = common_1.orderId(order);
    if (price === 0) {
        return {
            id,
            event: types_1.NormalizedMessageType.REMOVE,
            exchange: types_1.Exchange.ETHFINEX,
            network: options.network,
            base: options.pair.base,
            quote: options.pair.quote,
        };
    }
    return {
        id,
        event: types_1.NormalizedMessageType.SET,
        exchange: types_1.Exchange.ETHFINEX,
        network: options.network,
        base: options.pair.base,
        quote: options.pair.quote,
        order: common_1.normalizeOrder(options, order, id),
    };
};
const normalizeSnapshotEvent = (options, orders) => {
    const processed = orders
        .filter(([, price]) => price !== 0)
        .map(order => {
        const id = common_1.orderId(order);
        return common_1.normalizeOrder(options, order, id);
    });
    return {
        event: types_1.NormalizedMessageType.SNAPSHOT,
        exchange: types_1.Exchange.ETHFINEX,
        network: options.network,
        base: options.pair.base,
        quote: options.pair.quote,
        orders: processed,
    };
};
exports.watch = (options) => {
    const ws$ = new Rx.Observable(observer => {
        const open$ = new Rx.Subject();
        const close$ = new Rx.Subject();
        const ws$ = webSocket_1.webSocket({
            WebSocketCtor: isomorphic_ws_1.default,
            closeObserver: close$,
            openObserver: open$,
            url: getWebsocketUrl(options),
        });
        ws$.next(subscribeMessage(options));
        open$.subscribe(() => {
            debug.log('Opening connection.');
        });
        close$.subscribe(() => {
            debug.log('Closing connection.');
        });
        return ws$.subscribe(observer);
    }).pipe(operators_1.share());
    const messages$ = ws$.pipe(operators_1.filter(R.is(Array)), operators_1.map(R.nth(1)));
    const snapshots$ = messages$.pipe(operators_1.filter(R.compose(R.is(Array), R.head)), operators_1.map(orders => normalizeSnapshotEvent(options, orders)));
    const updates$ = messages$.pipe(operators_1.filter(R.compose(R.is(Number), R.head)), operators_1.map(order => normalizeOrderEvent(options, order)));
    return Rx.merge(snapshots$, updates$).pipe(operators_1.tap(event => debug.log('Source event: %e', event)), cleanEvents_1.cleanEvents(), operators_1.tap(event => debug.log('Output event: %e', event)));
};
