"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const protocol_1 = require("@melonproject/protocol");
const types_1 = require("../../types");
const token_math_1 = require("@melonproject/token-math");
const exchangePath = ['thirdPartyContracts', 'exchanges', 'matchingMarket'];
const contractPath = ['melonContracts', 'adapters', 'matchingMarketAccessor'];
exports.fetch = (options) => __awaiter(this, void 0, void 0, function* () {
    const environment = options.environment;
    const deployment = environment.deployment;
    const contract = R.path(contractPath, deployment);
    const exchange = R.path(exchangePath, deployment);
    const bidsCall = protocol_1.getActiveOasisDexOrders(environment, contract, {
        targetExchange: exchange.toString(),
        sellAsset: options.pair.quote.address.toString(),
        buyAsset: options.pair.base.address.toString(),
    });
    const asksCall = protocol_1.getActiveOasisDexOrders(environment, contract, {
        targetExchange: exchange,
        sellAsset: options.pair.base.address,
        buyAsset: options.pair.quote.address,
    });
    const [bidsResponse, asksResponse] = yield Promise.all([bidsCall, asksCall]);
    const bids = bidsResponse.map((order) => {
        const key = `${types_1.Exchange.OASIS_DEX}:${types_1.OrderType.BID}:${order.id}`;
        const id = Buffer.from(key).toString('base64');
        const trade = token_math_1.createPrice(order.buy, order.sell);
        return {
            id,
            exchange: types_1.Exchange.OASIS_DEX,
            type: types_1.OrderType.BID,
            trade,
            original: {
                id: order.id,
            },
        };
    });
    const asks = asksResponse.map((order) => {
        const key = `${types_1.Exchange.OASIS_DEX}:${types_1.OrderType.ASK}:${order.id}`;
        const id = Buffer.from(key).toString('base64');
        const trade = token_math_1.createPrice(order.sell, order.buy);
        return {
            id,
            exchange: types_1.Exchange.OASIS_DEX,
            type: types_1.OrderType.ASK,
            trade,
            original: {
                id: order.id,
            },
        };
    });
    return [].concat(bids, asks);
});
