"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const debug = __importStar(require("../debug"));
const rxjs_1 = require("rxjs");
const types_1 = require("../types");
exports.cleanEvents = (initial = []) => (source) => {
    return new rxjs_1.Observable(observer => {
        let orders = initial;
        return source.subscribe(message => {
            if (message.event === types_1.NormalizedMessageType.SNAPSHOT) {
                const snapshot = message;
                if (!orders || !orders.length) {
                    observer.next(message);
                }
                else {
                    snapshot.orders.forEach(current => {
                        if (!orders.find(R.equals(current))) {
                            observer.next({
                                event: types_1.NormalizedMessageType.SET,
                                exchange: snapshot.exchange,
                                network: snapshot.network,
                                base: snapshot.base,
                                quote: snapshot.quote,
                                id: current.id,
                                order: current,
                            });
                        }
                    });
                    orders.forEach(current => {
                        if (!snapshot.orders.find(R.propEq('id', current.id))) {
                            observer.next({
                                event: types_1.NormalizedMessageType.REMOVE,
                                exchange: snapshot.exchange,
                                network: snapshot.network,
                                base: snapshot.base,
                                quote: snapshot.quote,
                                id: current.id,
                            });
                        }
                    });
                }
                orders = snapshot.orders;
            }
            else if (message.event === types_1.NormalizedMessageType.SET) {
                const add = message;
                if (!orders.find(R.equals(add.order))) {
                    orders.push(add.order);
                    observer.next(add);
                }
                else {
                    debug.log('Filtered event: Set order %s.', add.id);
                }
            }
            else if (message.event === types_1.NormalizedMessageType.REMOVE) {
                const remove = message;
                const before = orders.length;
                orders = orders.filter(R.compose(R.not, R.propEq('id', remove.id)));
                if (orders.length !== before) {
                    observer.next(remove);
                }
                else {
                    debug.log('Filtered event: Remove order %s.', remove.id);
                }
            }
        }, observer.error, observer.complete);
    });
};
